#include "StdAfx.h"
#include "GlobalVariable.h"
#include <stdint.h>
#include <limits.h>

CGlobalVariable::CGlobalVariable()
{
	::InitializeCriticalSection(&m_Guard);
	m_cRef = 0;
	m_IntMap.clear();
}

CGlobalVariable::~CGlobalVariable()
{
	RemoveAllVariables();
	::DeleteCriticalSection(&m_Guard);
}

HRESULT __stdcall CGlobalVariable::QueryInterface( const IID& iid, void** ppv )
{
	if (iid == IID_IUnknown)
	{
		*ppv = static_cast<IGVData*>(this);
	}
	else if (iid == IID_IGVData)
	{
		*ppv = static_cast<IGVData*>(this);
	}
	else
	{
		*ppv = NULL;
		return E_NOINTERFACE;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef();
	return S_OK;
}

ULONG __stdcall CGlobalVariable::AddRef()
{
	return InterlockedIncrement(&m_cRef);
}


ULONG __stdcall CGlobalVariable::Release()
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this;
		return 0;
	}
	return m_cRef;
}

bool CGlobalVariable::SetDataInt( CString name, int _value )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("i"));
	if(!ContainsKeyInIntMap(nName))
	{
		IntVariables _i;
		_i.first = CString(nName);
		UnionDATATYPE _u;
		ZeroMemory(&_u, sizeof(UnionDATATYPE));
		_u.i = _value;

		_i.second = _u;
		m_IntMap.push_back(_i);
		return true;
	}
	else 
	{
		vector<IntVariables>::iterator _el = GetElement(nName);
		_el->second.i = _value;
		return true;
	}
	return false;
}

int CGlobalVariable::GetDataInt( CString name )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("i"));
	if(ContainsKeyInIntMap(nName))
	{
		vector<IntVariables>::iterator _i =  GetElement(nName);
		return _i->second.i;
	}
	return INT32_MAX;
}

bool CGlobalVariable::SetDataLong( CString name, long _value )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("l"));
	if(!ContainsKeyInIntMap(nName))
	{
		IntVariables _i;
		_i.first = CString(nName);
		UnionDATATYPE _u;
		ZeroMemory(&_u, sizeof(UnionDATATYPE));
		_u.l = _value;

		_i.second = _u;
		m_IntMap.push_back(_i);
		return true;
	}
	else 
	{
		vector<IntVariables>::iterator _el = GetElement(nName);
		_el->second.l = _value;
		return true;
	}
	
	return false;
}

long CGlobalVariable::GetDataLong( CString name )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("l"));
	if(ContainsKeyInIntMap(nName))
	{
		vector<IntVariables>::iterator _i =  GetElement(nName);
		return _i->second.l;
	}
	return LONG_MAX;
}

bool CGlobalVariable::SetDataFloat( CString name, float _value )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("f"));
	if(!ContainsKeyInIntMap(nName))
	{
		IntVariables _i;
		_i.first = CString(nName);
		UnionDATATYPE _u;
		ZeroMemory(&_u, sizeof(UnionDATATYPE));
		_u.f = _value;

		_i.second = _u;
		m_IntMap.push_back(_i);
		return true;
	}
	else 
	{
		vector<IntVariables>::iterator _el = GetElement(nName);
		_el->second.f = _value;
		return true;
	}
	return false;
}

float CGlobalVariable::GetDataFloat( CString name )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("f"));
	if(ContainsKeyInIntMap(nName))
	{
		vector<IntVariables>::iterator _i =  GetElement(nName);
		return _i->second.f;
	}
	return FLT_MAX;
}

bool CGlobalVariable::SetDataDouble( CString name, double _value )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("d"));
	if(!ContainsKeyInIntMap(nName))
	{
		IntVariables _i;
		_i.first = CString(nName);
		UnionDATATYPE _u;
		ZeroMemory(&_u, sizeof(UnionDATATYPE));
		_u.d = _value;

		_i.second = _u;
		m_IntMap.push_back(_i);
		return true;
	}
	else 
	{
		vector<IntVariables>::iterator _el = GetElement(nName);
		_el->second.d = _value;
		return true;
	}
	return false;
}

double CGlobalVariable::GetDataDouble( CString name )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("d"));
	if(ContainsKeyInIntMap(nName))
	{
		vector<IntVariables>::iterator _i =  GetElement(nName);
		return _i->second.d;
	}
	return DBL_MAX;
}

bool CGlobalVariable::SetDataString( CString name, CString _value )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("s"));
	if(!ContainsKeyInIntMap(nName))
	{
		IntVariables _i;
		_i.first = CString(nName);
		UnionDATATYPE _u;
		ZeroMemory(&_u, sizeof(UnionDATATYPE));

		if(MAXSTRINGLENGTH > _value.GetLength())
		{
			for(int i = 0; i <  _value.GetLength(); i++)
			{
				_u.wch[i] = _value[i];
			}

			_i.second = _u;
			m_IntMap.push_back(_i);
			return true;
		}	
	}
	else 
	{
		vector<IntVariables>::iterator _el = GetElement(nName);
		if(MAXSTRINGLENGTH > _value.GetLength())
		{
			ZeroMemory(&_el->second, sizeof(UnionDATATYPE));
			for(int i = 0; i <  _value.GetLength(); i++)
			{
				_el->second.wch[i] = _value[i];
			}
			return true;
		}
	}
	return false;
}

CString CGlobalVariable::GetDataString( CString name )
{
	Locker locker(m_Guard);
	CString nName = GetKey(name, + _T("s"));
	if(ContainsKeyInIntMap(nName))
	{
		vector<IntVariables>::iterator _i =  GetElement(nName);
		return CString(_i->second.wch);
	}
	return _T("");
}

bool CGlobalVariable::ContainsKeyInIntMap( CString key )
{
	vector<IntVariables>::iterator _begin = m_IntMap.begin();
	while(_begin != m_IntMap.end())
	{
		IntVariables _p = *_begin;
		if(_p.first == key)
			return true;

		_begin++;
	}
	return false;
}

bool CGlobalVariable::EraceElement( vector<IntVariables>::iterator _e)
{
	Locker lock(m_Guard);
	m_IntMap.erase(_e);
	return true;
}

vector<IntVariables>::iterator CGlobalVariable::GetElement( CString Name )
{
	vector<IntVariables>::iterator m_begin = m_IntMap.begin();
	while(m_begin != m_IntMap.end())
	{
		IntVariables v = *m_begin;
		if(v.first == Name)
			return m_begin;
		
		m_begin++;
	}
	return m_IntMap.end();
}

bool CGlobalVariable::RemoveAllVariables()
{
	Locker locker(m_Guard);
	m_IntMap.clear();
	return true;
}

CString CGlobalVariable::GetKey( CString Name, CString Prefix )
{
	 return Name + _T("_") + Prefix;
}
