#include "stdafx.h"
#include "DataConvertor.h"
#include "FileWriter.h"



FileWriter::FileWriter(CString logfilename, bool append)
{
	
	HANDLE hFile; 

	if(!append)
	{
		hFile = CreateFile(logfilename,    //  
		GENERIC_WRITE,          //   
		FILE_SHARE_READ,       //   
		NULL,                  //   
		CREATE_ALWAYS,         //  
		FILE_ATTRIBUTE_NORMAL, //  
		NULL);                 //   
	}
	else
	{
		hFile = CreateFile(logfilename,    //  
			GENERIC_WRITE,          //   
			FILE_SHARE_READ,       //   
			NULL,                  //   
			OPEN_ALWAYS,
			FILE_ATTRIBUTE_NORMAL, //  
			NULL);

		
		if(hFile == INVALID_HANDLE_VALUE && GetLastError() == ERROR_FILE_NOT_FOUND)
		{
			hFile = CreateFile(logfilename,    //  
				GENERIC_WRITE,          //   
				FILE_SHARE_READ,       //   
				NULL,                  //   
				CREATE_ALWAYS,         //  
				FILE_ATTRIBUTE_NORMAL, //  
				NULL);  
		}
		else
		{
			SetFilePointer(hFile, 0, NULL, FILE_END);
		}
	}

	if (hFile == INVALID_HANDLE_VALUE) 
	{ 
		assert(false);
	}

	mlogFileHandle = hFile;
	canCloseHandle = true;
}

FileWriter::FileWriter( HANDLE hFile )
{
	if (hFile == INVALID_HANDLE_VALUE) 
	{ 
		assert(false);
	}
	mlogFileHandle = hFile;
	canCloseHandle = false;
}

void FileWriter::SaveText(CString logStr)
{
	const WCHAR * data = logStr;
	LPCVOID buffer = (LPCVOID) data;
	DWORD countButesWriten = 0;
	DWORD countButesToWrite = logStr.GetLength() * sizeof(WCHAR);
	BOOL result = false;
	result = WriteFile(mlogFileHandle, buffer, countButesToWrite, &countButesWriten, NULL);
	assert(result);
}

#define  ilBufferSize 15
#define  dBufferSize 308


void FileWriter::SaveText(int ivalue)
{
	CString sValue = DataConvertor::ToString(ivalue);
	SaveText(sValue);
}

void FileWriter::SaveText(long lvalue)
{ 
	CString sValue = DataConvertor::ToString(lvalue);
	SaveText(sValue);
} 

void FileWriter::SaveText(double dvalue, int numOfDigits)
{
	CString sValue = DataConvertor::ToString(dvalue, numOfDigits);
	SaveText(sValue);
}

void FileWriter::NewLine(void)
{
	CString nl = _T("\r\n");
	SaveText(nl);
}

void FileWriter::CreateDirectory(CString newDirName)
{
	BOOL res = 0;
	res = ::CreateDirectory( newDirName, NULL);
	if(res != 0)
	{
		DWORD err = GetLastError();
		switch(err)
		{
		case ERROR_ALREADY_EXISTS:
				throw("ERROR_ALREADY_EXISTS");
			break;

		case ERROR_PATH_NOT_FOUND:
				throw("ERROR_PATH_NOT_FOUND");
			break;

			default: break;
		}
	}
}

FileWriter::~FileWriter(void)
{
	if(canCloseHandle)
		if(mlogFileHandle != INVALID_HANDLE_VALUE)  
			if(mlogFileHandle != NULL)
			{
				BOOL result = FlushFileBuffers(mlogFileHandle);
				assert(result);

				BOOL res = CloseHandle(mlogFileHandle);
				assert(res != 0);
			}
}

void FileWriter::Close()
{
	BOOL result = FlushFileBuffers(mlogFileHandle);
	assert(result);

	BOOL res = CloseHandle(mlogFileHandle);
	assert(res != 0);
	mlogFileHandle = INVALID_HANDLE_VALUE;
}

void FileWriter::SaveBinaryData(void * data, int dataCount)
{
	LPCVOID buffer = (LPCVOID) data;
	DWORD countButesWriten = 0;
	DWORD countButesToWrite = dataCount;
	BOOL result = false;
	result = WriteFile(mlogFileHandle, buffer, countButesToWrite, &countButesWriten, NULL);
	assert(result);
}

HANDLE FileWriter::getFileHadle()
{
	return mlogFileHandle;
}

void FileWriter::SaveBinary( int val )
{
	VARIANT v;
	::VariantInit(&v);
	v.iVal = val;

	SaveBinaryData(&v.iVal, sizeof(int));
}

void FileWriter::SaveBinary( long val )
{
	VARIANT v;
	::VariantInit(&v);
	v.lVal = val;
	SaveBinaryData(&v.lVal, sizeof(long));
}

void FileWriter::SaveBinary( float val )
{
	VARIANT v;
	::VariantInit(&v);
	v.fltVal = val;
	SaveBinaryData(&v.fltVal, sizeof(float));
}

void FileWriter::SaveBinary( double val )
{
	VARIANT v;
	::VariantInit(&v);
	v.dblVal = val;
	SaveBinaryData(&v.dblVal, sizeof(double));
}

void FileWriter::SaveBinary( bool val )
{
	VARIANT v;
	::VariantInit(&v);
	v.bVal= val;
	SaveBinaryData(&v.bVal, sizeof(bool));
}
